//-------------------------------------------------------------------------
// Global Data
//-------------------------------------------------------------------------
#ifndef DATA_DRAIN
#define DATA_DRAIN

#include "_SkinnedMesh.fx"

// SKin
float4x4 ViewProjection;
float4 LightVector;

float4x4 World;
float4x4 WorldViewProjection;
float3 Light;
float Width;
float Height;
float NormalScale;
float PixelScale;

//---------------------------------------------------------------------------
// VertexShader
//----------------------------------------------------------------------------
void SkinMeshExplosionVertexShaser ( VS_SKINNING Input , uniform int NumBones,
									out float4 Pos : POSITION, 
									out float4 VertexColor : COLOR0,
									out float2 MeshUV : TEXCOORD0,
									out float2 PixelXY : TEXCOORD1
)
{
	// ]̂݃XLjO
	VS_SKINNINGOUTPUT Output = Vertex_TransformNormal( Input , NumBones  );
	// _Wˉeϊ
	float4 WorldViewProjection_Pos = mul( float4( Output.Pos.xyz , 1.0f )  , ViewProjection );
	
	
	float4 WorldViewProjection_Normal = float4(mul( Output.Normal, (float3x3)ViewProjection), 0 );
	
	// _Ɉړ
	Pos = WorldViewProjection_Pos + normalize( WorldViewProjection_Normal ) * NormalScale * WorldViewProjection_Pos.w;
	
	// eNX`W
	MeshUV = Output.TextureCoord;
	
	// _F
	float3 w_normal = normalize( normalize(Output.Normal ));
	VertexColor = max( 0, dot( w_normal, -LightVector) ) * MaterialDiffuse + MaterialAmbient;
	VertexColor.a = MaterialDiffuse.a;
	
	// sNZgU̎Zo
	PixelXY = Pos.xy * 0.5f + 0.5f;	
}



void StaticMeshExplosionVertexShaser (
	float4 pos : POSITION,
	float3 normal : NORMAL,
	float2 mesh_uv : TEXCOORD0, 
	out float4 Pos : POSITION, 
	out float4 Col : COLOR0,
	out float2 MeshUV : TEXCOORD0,
	out float2 PixelXY : TEXCOORD1
) {
	// _W
	float4 wvp_pos=mul( pos, WorldViewProjection );
	
	/*wvp_pos.x -= 5.5f;
	wvp_pos.y -= 5.5f;
	wvp_pos.z -= 5.5f;
	*/
	
	float4 wvp_normal=float4(mul(normal, (float3x3)WorldViewProjection), 0);
	Pos = wvp_pos + normalize(wvp_normal) * NormalScale * wvp_pos.w;
	
	// eNX`W
	MeshUV = mesh_uv;
	
	// _F
	float3 w_normal = normalize(mul(normal, (float3x3)World));
	Col = max( 0, dot( w_normal, -Light) ) * MaterialDiffuse + MaterialAmbient;
	Col.a = MaterialDiffuse.a;
	
	// sNZgU̎Zo
	PixelXY = Pos.xy * 0.5f + 0.5f;
}

// PixcelShader
float4 ExplosionPS(
	float4 Col : COLOR, 
	float2 MeshUV : TEXCOORD0,
	float2 PixelXY : TEXCOORD1
) : COLOR 
{
	float4 color;
	float x = PixelXY.x * Width , y = PixelXY.y * Height;
	
	
	// Pixcelʓɂꍇ
	if ( abs( x % PixelScale ) < 1.0f && abs( y % PixelScale ) < 1.0f )
	{
		// ColorZo
		color = tex2D( MeshSampler, MeshUV ) * Col;
	}
	else 
	{
		//ʊȌꍇ̓J[
		color=float4( 0, 0, 0, 0 );
	}
	
	return color;
}


// {[ݒ
int CurNumBones = 2;
VertexShader vsDataDrainArray[4] = { 
				compile vs_2_0 SkinMeshExplosionVertexShaser(1), 
                            compile vs_2_0 SkinMeshExplosionVertexShaser(2),
                            compile vs_2_0 SkinMeshExplosionVertexShaser(3),
                            compile vs_2_0 SkinMeshExplosionVertexShaser(4)
                          };

//-----------------------------------------------------------------------
// eNjbN
//------------------------------------------------------------------------
technique SkinDataDrain 
{
	pass Pass0
	{
		// wʃJO
		CullMode = NONE;

	        VertexShader = ( vsDataDrainArray[CurNumBones] );
       			
		// Z
		SrcBlend = SRCALPHA;
		DestBlend = ONE;
		
		PixelShader=compile ps_2_0 ExplosionPS();
	}
}
//-----------------------------------------------------------------------
// eNjbN
//------------------------------------------------------------------------
technique StaticDataDrain 
{
	pass P0
	{
		// wʃJO
		CullMode = NONE;
		
		VertexShader=compile vs_2_0 StaticMeshExplosionVertexShaser();
		
		// Z
		SrcBlend = SRCALPHA;
		DestBlend = ONE;
		
		PixelShader=compile ps_2_0 ExplosionPS();
	}
}

#endif
